#include <avr/io.h>
#include "lcd-routines.h"
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>		
#include <inttypes.h>
#include <util/delay.h>  


/* ====================================================================== */
/* Defines zur Erhhung der Lesbarkeit                                    */
/* ====================================================================== */

#define fosc 12000000

typedef struct
{
    volatile unsigned char Sekunden;
    volatile unsigned char Minuten;
    volatile unsigned char Stunden;
    volatile unsigned char Tag;
    volatile unsigned char Monat;
    volatile unsigned int Jahr;
} datestruct;


/* ====================================================================== */
/* Variablendefinitionen                                                  */
/* ====================================================================== */
		uint8_t menue_cnt = 0;				//Menschritte
		uint8_t Timer_stellen = 0;		//Uhrzeit einstellen - berspringen
		uint8_t Timer_anhalten = 0;		//Timer anhalten, wenn Timer_anhalten = 1
volatile unsigned int adcwert;		//AD-Wandler Wert
volatile datestruct Uhrzeit;
volatile unsigned char letzterTag[2][12] = {{31,28,31,30,31,30,31,31,30,31,30,31},
                                            {31,29,31,30,31,30,31,31,30,31,30,31}};
volatile unsigned char Schaltjahr = 0;

void Uhrzeitfunktion (void);
void Uhrzeitfunktion_stellen (void);

/* ====================================================================== */
/* Initialisierungsroutinen                                               */
/* ====================================================================== */
//Timer-Initialisieren
void init_Timer1(void)
{
	TIMSK = (1<<OCIE1A);							//Vergleichsinterrupt COMPA erlauben
	sei();														//Interrupts ermglichen
	TCCR1B |= (1<<WGM12); 						//CTC Modus --> Vergleichsmodus mit OCR1A als Vergleicher
	TCCR1B |= (1<<CS12) | (1<<CS10);	//Prescaler 1024 bei |= (1<<CS12) | (1<<CS10); 256 bei |= (1<<CS12)
	OCR1A = 5858*2;										//OCR1A Wert, Vergleichswert COMPA (ISR (TIMER1_COMPA_vect)) ZEHNTELSEKUNDE
  //5858 	 = halber Sekundentak = 2 Hz
	//5858*2 = Sekundentakt = 1 Hz
}

//ADC-Initialisieren

void init_ADC(void)
{
ADMUX |= (0<<REFS1) | (1<<REFS0);   //VCC als Referenz
ADCSRA = (1<<ADEN) | (0<<ADPS2) | (1<<ADPS1) | (1<< ADPS0);  //Frequenzvorteiler auf 8
ADMUX |= (1<<MUX1); //ADC2 MUX0 = 0, MUX1 = 1, MUX2 = 0; MUX3 = 0;
//Dummy Readyout ADC
ADCSRA |= (1<<ADSC);        
while (ADCSRA & (1<<ADSC) ) {
      ;
    }
adcwert = ADCW;
}
/* ====================================================================== */
/* Displayausgabe                                                         */
/* ====================================================================== */

void display_clock( void )
{
  lcd_command( 0x80 );				// line 1
  lcd_data( Uhrzeit.Stunden / 10 + '0');
  lcd_data( Uhrzeit.Stunden % 10 + '0');
  lcd_data( ':' );
  lcd_data( Uhrzeit.Minuten / 10 + '0');
  lcd_data( Uhrzeit.Minuten % 10 + '0');
  lcd_data( ':' );
  lcd_data( Uhrzeit.Sekunden / 10 + '0');
  lcd_data( Uhrzeit.Sekunden % 10 + '0');

  lcd_command( 0xC0 );				// line 2
  lcd_data( Uhrzeit.Tag / 10 + '0');
  lcd_data( Uhrzeit.Tag % 10 + '0');
  lcd_data( '.' );
  lcd_data( Uhrzeit.Monat / 10 + '0');
  lcd_data( Uhrzeit.Monat % 10 + '0');
  lcd_data( '.' );
  lcd_data( '2' );
  lcd_data( '0' );
  lcd_data( Uhrzeit.Jahr / 10 + '0');
  lcd_data( Uhrzeit.Jahr % 10 + '0');
}

/* ====================================================================== */
/* Entprellfunktion fr beliebige Taster                                  */
/* ====================================================================== */
static uint8_t debounce (volatile uint8_t *port, uint8_t pin)
	{
    if ( !(*port & (1 << pin)) )
    {
        		/* Pin wurde auf Masse gezogen, 100ms warten   */
        _delay_ms(50);   // Maximalwert des Parameters an _delay_ms 
        _delay_ms(50);   // beachten, vgl. Dokumentation der avr-libc
        if ( *port & (1 << pin) )
        {
            /* Anwender Zeit zum Loslassen des Tasters geben */
            _delay_ms(50);
            _delay_ms(50); 
            return 1;
        }
    }
    return 0;
	}


/* ====================================================================== */
/* Hauptschleife mit Initialisierungen                                    */
/* ====================================================================== */

int main (void)
{
  // Variablen vereinbaren / initialisieren
  init_Timer1();				//Timer/Uhr initialisieren
	init_ADC();						//AD Wandler initialisieren
	lcd_init();						//LCD initialisieren

  // Interrupts freigeben
  sei();

	//Statusled und Eingangstaster definieren
 		DDRB = 0b11111111;
		PORTB |= (1 << PB0);						// PB0 als Statusled

		DDRC &= ~( 1 << PC0 );        	// PIN PC0 auf Eingang Taster 
    PORTC |= ( 1 << PC0 );        	// Pullup-Widerstand aktivieren

    DDRC &= ~( 1 << PC1 );        	// PIN PC1 auf Eingang Taster  
    PORTC |= ( 1 << PC1 );        	// Pullup-Widerstand aktivieren 

    DDRC &= ~( 1 << PC2 );        	// PIN PC1 auf Eingang Taster  
  //  PORTC |= ( 1 << PC2 );        	// Pullup-Widerstand aktivieren 



	//Hauptschleife
	while (1)
	{
		 if (debounce(&PINC, PC0))			//Entprellfunktion PC0
	    {
	        menue_cnt++;							//Falls Taster an PIN PC0 gedrueckt zhle im Men +1 und lsche Display beim Umschalten    */
					lcd_clear ();
	    }


	switch (menue_cnt)
			{



		/*Startbildschirm*/

			case 0:
							lcd_setcursor (1,1);
							lcd_string ("Clock Setup");
							lcd_setcursor (2,2);
							lcd_string ("Press Button");

							if (Timer_stellen >= 1)
							menue_cnt = 11;

							break;



		/*UHR einstellen*/

			//MINUTEN einstellen
			case 1:	
	 lcd_command( LCD_SET_FUNCTION |
                LCD_FUNCTION_1LINE);
															
							lcd_setcursor (0,1);
							lcd_string ("Set Minutes");	
								_delay_ms (700);
							lcd_clear ();
							display_clock();   
							menue_cnt = 2;
							break;
			case 2:
						if (PINC & (1<<PC1))
								{
								Uhrzeit.Sekunden = 0;
								Timer_anhalten = 1;	
												
								++Uhrzeit.Minuten;
								Uhrzeitfunktion_stellen ();

								_delay_ms (20);
								}
						else 
								Timer_anhalten = 0;




							display_clock();	
							break;

			//STUNDEN einstellen
			case 3:	
							
							lcd_setcursor (0,1);
							lcd_string ("Set Hour");	
								_delay_ms (700);
							lcd_clear ();
							display_clock();
							menue_cnt = 4;
							break;
			case 4:
						if (PINC & (1<<PC1))
								{
								Timer_anhalten = 1;	
												
								++Uhrzeit.Stunden;
								Uhrzeitfunktion_stellen ();

								_delay_ms (70);
								}
						else 
								Timer_anhalten = 0;

							display_clock();	
							break;


			//TAG einstellen
			case 5:	
	 lcd_command( LCD_SET_FUNCTION |
                LCD_FUNCTION_2LINE);	
														
							lcd_setcursor (0,1);
							lcd_string ("Set Day");	
								_delay_ms (700);
							lcd_clear ();
							display_clock();
							menue_cnt = 6;
							break;
			case 6:
						if (PINC & (1<<PC1))
								{
								Timer_anhalten = 1;	
												
								++Uhrzeit.Tag;
								Uhrzeitfunktion_stellen ();

								_delay_ms (70);
								}
						else 
								Timer_anhalten = 0;

							display_clock();	
							break;

			//MONAT einstellen
			case 7:	
							
							lcd_setcursor (0,1);
							lcd_string ("Set Month");	
								_delay_ms (700);
							lcd_clear ();
							display_clock();
							menue_cnt = 8;
							break;
			case 8:
						if (PINC & (1<<PC1))
								{
								Timer_anhalten = 1;	
												
								++Uhrzeit.Monat;
								Uhrzeitfunktion_stellen();

								_delay_ms (70);
								}
						else 
								Timer_anhalten = 0;

							display_clock();	
							break;

			//JAHR einstellen
			case 9:	
							
							lcd_setcursor (0,1);
							lcd_string ("Set Year");	
								_delay_ms (700);
							lcd_clear ();
							display_clock();
							menue_cnt = 10;
							break;

			case 10:
						if (PINC & (1<<PC1))
								{
								Timer_anhalten = 1;	
												
								++Uhrzeit.Jahr;
								Uhrzeitfunktion_stellen();

								_delay_ms (70);
								}
						else 
								Timer_anhalten = 0;

							display_clock();
							Timer_stellen = 1;	
							break;



		/*Hauptmen*/

			case 11:
							lcd_setcursor (4,1);
							lcd_string ("Home");
							lcd_setcursor (2,2);
							lcd_string ("Press Button");	
							break;


		/*Uhr anzeigen*/

			//Startup Uhrzeit
			case 12:	
							lcd_setcursor (4,1);
							lcd_string ("Time");	
								_delay_ms (1000);
							lcd_clear ();
							display_clock();
							menue_cnt = 13;
							break;

			//Display Uhrzeit
			case 13:
							Timer_anhalten = 0;
							display_clock();
							break;


		/*ADC-Anzeige an RC2 bzw. ADC2*/

			//Startup ADC
			case 14:
							lcd_setcursor (1,1);
							lcd_string ("AD Converter");	
								_delay_ms (1000);
							lcd_clear ();
							display_clock();
							menue_cnt = 15;
							break;
			
			case 15:
							lcd_setcursor (1,1);
							lcd_string ("AD Convert");
							ADCSRA |= (1<<ADSC);
							while (ADCSRA & (1<<ADSC) ) {
	      					;
	    					}
				//ADCW-Wert in STRING wandeln
							adcwert = ADCW;
							char buf[4];
							itoa(adcwert,buf,10);
				//Ergebnis auf Display darstellen
							lcd_clear ();
							lcd_setcursor (1,1);
							lcd_string ("AD Converter");
							lcd_setcursor(2,2);
							lcd_string(buf);
							lcd_setcursor (8,2);
							lcd_string ("[/]");
							break;




			case 16:
							menue_cnt = 11;
							break;

			default:
							menue_cnt = 11;
							break;
		}
	}
return 0;
}



/* ====================================================================== */
/* Interrupt Service Routinen/ Uhrzeit                                    */
/* ====================================================================== */

ISR (TIMER1_COMPA_vect)		//Timer/Counter1 overflow
{ 
		if (Timer_anhalten != 1)	//Wenn Stopbit nicht (1) gesetzt, luft Timer1
		{
				Uhrzeitfunktion();
		}
}

void Uhrzeitfunktion (void)
{		
		 if (++Uhrzeit.Sekunden >59)
		 {
	    Uhrzeit.Sekunden = 0;
	    if (++Uhrzeit.Minuten > 59)
	    {
	      Uhrzeit.Minuten = 0;
	      if (++Uhrzeit.Stunden >23)
	      {
	        Uhrzeit.Stunden = 0;
	        if (++Uhrzeit.Tag > letzterTag[Schaltjahr][Uhrzeit.Monat-1])
	        {
	          Uhrzeit.Tag = 1;
	          if (++Uhrzeit.Monat >12)
	          {
	            Uhrzeit.Monat = 1;
	            ++Uhrzeit.Jahr;
	            if ((Uhrzeit.Jahr % 4 == 0) && ((Uhrzeit.Jahr % 100 != 0) || (Uhrzeit.Jahr % 400 == 0)))
	            {
	              Schaltjahr = 0;
	            }
	            else
	            {
	              Schaltjahr = 1;
	            }
	          }
	        }
	      }
			}
		}
}

void Uhrzeitfunktion_stellen (void)
{		
		if (Timer_anhalten != 1)	//Wenn Stopbit nicht (1) gesetzt, luft Timer1
		{
				Uhrzeit.Sekunden++;
		}

		  if (Uhrzeit.Sekunden >59)
		  {
		    Uhrzeit.Sekunden = 0;
				++Uhrzeit.Minuten;
			}

	    if (Uhrzeit.Minuten > 59)
	    {
	      Uhrzeit.Minuten = 0;
				Uhrzeit.Stunden++;
			}

	    if (Uhrzeit.Stunden >23)
	    {
	    Uhrzeit.Stunden = 0;
			Uhrzeit.Tag++;
			}

	    if (Uhrzeit.Tag > letzterTag[Schaltjahr][Uhrzeit.Monat-1])
	    {
	    Uhrzeit.Tag = 1;
			Uhrzeit.Monat++;
			}

	    if (Uhrzeit.Monat >12)
	    {
	    Uhrzeit.Monat = 1;
	    ++Uhrzeit.Jahr;
			}

	    if ((Uhrzeit.Jahr % 4 == 0) && ((Uhrzeit.Jahr % 100 != 0) || (Uhrzeit.Jahr % 400 == 0)))
	    {
		  Schaltjahr = 0;
	    }
	    else
	    {
	     Schaltjahr = 1;
	    }
	   
}
